# install.packages("shiny", dependencies = T)
# install.packages("shinythemes", dependencies = T)
# install.packages("shinyWidgets", dependencies = T)

library(shiny)
library(shinythemes)
library(shinyWidgets)
library(ggplot2)
library(scales)

ui <- shinyUI(navbarPage(theme = shinytheme("cerulean"),"DESCRIPCIÓN DE 2 VARIABLES",
                         tabPanel("Archivo",uiOutput("component1")),
                         tabPanel("Variables",uiOutput("component2")),
                         tabPanel("Dos variables categóricas", uiOutput("component3")),
                         tabPanel("Dos variables numéricas", uiOutput("component4")),
                         tabPanel("Una variable numérica y una categórica", uiOutput("component5"))
))

server <- function(input, output) {
  
  output$component1 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        fileInput('file1', 'Lectura de fichero:',
                  accept = c(
                    'text/csv',
                    'text/comma-separated-values',
                    'text/tab-separated-values',
                    'text/plain',
                    '.csv',
                    '.tsv'
                  ),
                  buttonLabel = "Buscar fichero",
                  placeholder = "Ningún archivo seleccionado"
        ),
        radioButtons("header",
                     label="¿El archivo contiene el nombre de la variable?",
                     choices=list("Sí","No"),
                     selected="Sí"),
        radioButtons(inputId = 'sep', label='Separador de variables',
                     c(Espacio ='\ ',
                       "Tabulación"='\t',
                       Coma=',',
                       "Punto y Coma"=';'
                       ),
                     '\ '),
        radioButtons('quote', 'Formato',
                     c('Comilla Doble'='"',
                       'Comilla simple'="'",
                       'Nada'=''),
                     '"'),
        numericInput(inputId = "obs",label = "Número de observaciones a mostrar:",value=10),
        tags$hr()
      ),
      mainPanel(
        fluidRow(
          column(8,
                 h1("Fichero de datos"),
                 p(strong("Sección Dos variables categóricas: "),em("tabla de contingencia y grafico de barras múltiples")),
                 p(strong("Sección Dos variables numéricas: "),em("tabla de contingencia, estadísticos, recta de regresión")),
                 p(strong("Sección Una variable numérica y una categórica: "),em(" diagrama de cajas múltiple, tabla con estadísticos"))
          ),
          column(3, br(),br(),img(src = "numeros.gif", height = 225, width = 225))
        ),
        h4("Las dimensiones del fichero escogido son:"),
        fluidRow(column(1,br()),
                 column(4,p("Número de observaciones:")),
                 column(3,verbatimTextOutput("dimensionesFilas",placeholder = T))
        ),
        fluidRow(column(1,br()),
                 column(4,p("Número de variables:")),
                 column(3,verbatimTextOutput("dimensionesColumnas",placeholder = T))
        ),
        h4("Primeros datos del fichero:"),
        tableOutput('contents')
      )
    )
  })
  
  output$component2 <- renderUI({
    sidebarLayout(
      sidebarPanel(
        h2("Eleccion de variables"),
        h3("Primera variable: ",style = "font-family: 'Source Sans Pro';"),
        numericInput(inputId = "var1",label = "Variable escogida:",value=1),
        radioButtons("tipo1",
                     label="Selecciona si la variable es numérica o cualitativa:",
                     choices=list("Cualitativa","Numérica"),
                     selected="Cualitativa"),
        conditionalPanel(condition = "input.tipo1 == 'Numérica'",        
                         radioButtons("tipoNumerica1",
                               label="Escoge el tipo de numérica:",
                               choices=list("Continua","Discreta"),
                               selected="Continua")),
        
        h3("Segunda variable: ",style = "font-family: 'Source Sans Pro';"),
        numericInput(inputId = "var2",label = "Variable escogida:",value=2),
        radioButtons("tipo2",
                     label="Selecciona si la variable es numérica o cualitativa:",
                     choices=list("Cualitativa","Numérica"),
                     selected="Cualitativa"),
        conditionalPanel(condition = "input.tipo2 == 'Numérica'",        
                         radioButtons("tipoNumerica2",
                                      label="Escoge el tipo de numérica:",
                                      choices=list("Continua","Discreta"),
                                      selected="Continua"))
      ),
      mainPanel(
        fluidRow(column(5,br(),br(),br(),numericInput(inputId = "obs1",label = "Número de observaciones a mostrar:",value=10)),
                 column(5,img(src = "goma.gif", height = 200, width = 300,align="center"))),
        fluidRow(column(4,        
                        h3("Datos variable:"),
                        tableOutput('contentsV1')
        ),
        column(4,
               h3("Datos variable:"),
               tableOutput('contentsV2')
        )
        )
      )
    )
  })
  
  output$component3 <- renderUI({
    sidebarLayout(
      sidebarPanel(width = 3,
        p(strong(p("Tabla de doble entrada o contingencia: ")),em("Tabla que hace referencia a dos variables")),br(),
        p(strong(p("Tabla de frecuencias marginales: ")),em("Tabla de frecuencia para cada variable")),br(),
        strong("Gráfico de barras múltiple"),br(),br(),
        strong(h4(em("Si se ha declarado el tipo de variable erróneamente los resultados puede que no sean válidos."))),br()
      ),
      mainPanel(
        wellPanel(style = "background: #FFFFFF",
                  conditionalPanel(condition = "input.tipo1 == 'Cualitativa' && input.tipo2 == 'Cualitativa'",
                     fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#A8D1FC",column(5, h3("Nombre variable 1: "),br(),
                                        h3("Nombre variable 2: ")),
                              column(5,br(),verbatimTextOutput("nombrev1"),br(),
                                            verbatimTextOutput("nombrev2"))
                     ),br(),
                     h4("Tabla de doble entrada:"),
                     tableOutput("view1"),
                     fluidRow(column(6,h4("Marginal de X"),
                                     tableOutput("marg11")),
                              column(6, h4("Marginal de Y"),
                                     tableOutput("marg21"))
                     ),
                     br(),
                     h4("Condicionada de X por Y"),
                     tableOutput("condX1"),br(),
                     h4("Condicionada de Y por X"),
                     tableOutput("condY1"),
                     h4("Gráfico de barras multiples"),
                     plotOutput(outputId = "barras"),br(),
                     h4("Gráfico de barras apiladas"),
                     plotOutput(outputId = "barrasApiladas"),br()
                 ),
                 conditionalPanel(condition = "input.tipo1 != 'Cualitativa' || input.tipo2 != 'Cualitativa'",
                                  h4("Para obtener resultados en esta sección las dos variables deben ser declaradas categóricas"))
        )
      )
    )
  })
  
  output$component4 <- renderUI({
    sidebarLayout(
      sidebarPanel(width = 3,
        p(strong(p("Tabla de doble entrada o contingencia: ")),em("Tabla que hace referencia a dos variables")),
        p(strong(p("Tabla de frecuencias marginales: ")),em("Tabla de frecuencia para cada variable")),
        p(strong(p("Tabla de distribución condicionada"))),
        p(strong(p("Coeficiente de correlación lineal de Pearson:")), em(" valor entre -1 y 1 que indica la intensidad y dirección de la relación entre X e Y")),
        p(strong(p("Covarianza:")),em(" valor que indica el grado de variación conjunta de dos variables")),
        p(strong(p("Recta de regresión:")),em(" línea recta que mejor se ajusta a la nube de puntos")),br(),
        strong(h4(em("Si se ha declarado el tipo de variable erróneamente los resultados puede que no sean válidos."))),br()
      ),
      mainPanel(
        wellPanel(style = "background: #FFFFFF",
            conditionalPanel(condition = "input.tipo1 == 'Numérica' && input.tipo2 == 'Numérica'",
                  fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#A8D1FC",column(5, h3("Nombre variable 1: "),br(),
                                  h3("Nombre variable 2: ")),
                           column(5,br(),verbatimTextOutput("nombrev3"),br(),
                                  verbatimTextOutput("nombrev4"))
                  ),br(),
                  textOutput("particionIntervalo1"),
                  if(input$tipoNumerica1 =="Continua"){
                    numericInput(inputId = "particionV1",label = " ",value=5)
                  },
                  if(input$tipoNumerica2 =="Continua"){
                    numericInput(inputId = "particionV2",label = " ",value=5)
                  },br(),
                  h4("Tabla de doble entrada:"),
                  tableOutput("view"),
                  br(),
                  fluidRow(column(6,h4("Marginal de X"),
                                  tableOutput("marg1")),
                           column(6, h4("Marginal de Y"),
                                  tableOutput("marg2"))
                           ),
                  br(),
                  h4("Condicionada de X por Y"),
                  tableOutput("condX"),br(),
                  h4("Condicionada de Y por X"),
                  tableOutput("condY"),
                  br(),
                  h4("Tabla de cálculos intermedios:"),
                  radioButtons("resultados",
                               label=" ",
                               choices=list("Mostrar","No mostrar"),
                               selected="No mostrar"),
                  tableOutput("tablaIntermedia"),
                  p("Media X:"),
                  verbatimTextOutput("mediaX"),
                  p("Media Y:"),
                  verbatimTextOutput("mediaY"),
                  p("Desviación Típica X:"),
                  verbatimTextOutput("desviacionX"),
                  p("Desviación Típica Y:"),
                  verbatimTextOutput("desviacionY"),
                  p("Covarianza:"),
                  verbatimTextOutput("cov"),
                  p("Coeficiente de correlación lineal de Pearson:"),
                  verbatimTextOutput("cor"),
                  p("Nube de puntos"),
                  plotOutput(outputId = "nube"),
                  br(),br(),br(),
                  p("Coeficientes de la ecuación de la recta de regresión:"),
                  verbatimTextOutput("regresionX"),
                  p("Gráfico de la recta de regresión Y sobre X:"),
                  plotOutput(outputId = "rectaX"),
                  radioButtons("modificarRecta",
                               label="¿Modificar ejes del gráfico?",
                               choices=list("Sí","No"),
                               selected="No"),
                  conditionalPanel(condition = "input.modificarRecta == 'Sí'",
                    fluidRow(column(5,numericInput(inputId = "infX",label = "Límite inferior del eje X :",value=0),
                                    numericInput(inputId = "infY",label = "Límite inferior del eje Y :",value=0)),
                             column(5,numericInput(inputId = "supX",label = "Límite superior del eje X :",value=2),
                                    numericInput(inputId = "supY",label = "Límite superior del eje Y :",value=2))),
                    plotOutput(outputId = "rectaXModif")
                  )
                  
            ),
            conditionalPanel(condition = "input.tipo1 != 'Numérica' || input.tipo2 != 'Numérica'",
                             h4("Para obtener resultados en esta sección las dos variables deben ser declaradas numéricas"))
            )
        )
    )
  })
  
  output$component5 <- renderUI({
    sidebarLayout(
      sidebarPanel(width = 3,
        h4("Tabla de estadísticos"),
        p("Estadísticos básicos con los que se construye el gráfico de cajas"),
        h4("Gráfico de cajas"),
        p("Útil para comparar."),br(),br(),
        strong(h4(em("Si se ha declarado el tipo de variable erróneamente los resultados puede que no sean válidos."))),br()
      ),
      mainPanel(
        wellPanel(style = "background: #FFFFFF",
                conditionalPanel(condition = "(input.tipo1 == 'Numérica' && input.tipo2 == 'Cualitativa') || (input.tipo2 == 'Numérica' && input.tipo1 == 'Cualitativa')",
                  fluidRow(style="border:1px inset;border-color:#458cc3;background-color:#A8D1FC",column(5, h3("Nombre variable 1: "),br(),
                                  h3("Nombre variable 2: ")),
                           column(5,br(),verbatimTextOutput("nombrev5"),br(),
                                  verbatimTextOutput("nombrev6"))
                  ),br(),
                  h3("Tabla de estadísticos"),
                  tableOutput("estadisticos"),
                  h3("Gráfico de cajas"),
                  radioButtons("orientacion",
                               label="Selecciona la orientación de las cajas:",
                               choices=list("Horizontal","Vertical"),
                               selected="Horizontal"),
                  plotOutput(outputId = "cajas")
                ),
                conditionalPanel(condition = "(input.tipo1 == 'Numérica' && input.tipo2 == 'Numérica')",
                                 h4("Para obtener resultados en esta sección una variable tiene que ser declarada numérica y la otra categórica")),
                conditionalPanel(condition = "(input.tipo1 == 'Cualitativa' && input.tipo2 == 'Cualitativa')",
                                 h4("Para obtener resultados en esta sección una variable tiene que ser declarada numérica y la otra categórica"))
        )
      )
    )
  })
  
  
  
  ########################### LECTURA DE DATOS #################################### 
  datasetInput <- reactive({
    req(input$file1)
    inFile <- input$file1
    if(input$header == "Sí"){
      a <- TRUE
    }else{ a <- FALSE}
    db <- read.csv(inFile$datapath, header = a,
                   sep = input$sep, quote = input$quote)
    db <- data.frame(db)
  })
  
  output$contents <- renderTable({
    if(input$obs <=0){
      NULL
    }else{
      head(datasetInput(),n=input$obs)
    }
  })
  
  # VARIABLE 1
  datasetInput1 <- reactive({
    db <- datasetInput()
    if(input$var1 <= 0){
      db <- NULL
    }else if(input$var1 > ncol(db)){
      db <- NULL
    }else{
      nombre <- names(datasetInput())
      db <-as.data.frame(db[,input$var1])
      colnames(db) <- nombre[input$var1]
      db
    }
  })
  
  output$contentsV1 <- renderTable({
    if(input$obs1 <=0){
      NULL
    }else{
      head(datasetInput1(),n=input$obs1)
    }
  })
  
  # VARIABLE 2
  datasetInput2 <- reactive({
    db <- datasetInput()
    if(input$var2 <= 0){
      db <- NULL
    }else if(input$var2 > ncol(db)){
      db <- NULL
    }else{
      nombre <- names(datasetInput())
      db <-as.data.frame(db[,input$var2])
      colnames(db) <- nombre[input$var2]
      db
    }
  })
  
  output$dimensionesFilas <- renderPrint({
    cat(nrow(datasetInput()))
  }) 
  
  output$dimensionesColumnas <- renderPrint({
    cat(ncol(datasetInput()))
  })
  
  output$contentsV2 <- renderTable({
    if(input$obs1 <=0){
      NULL
    }else{
      head(datasetInput2(),n=input$obs1)
    }
  })
  
  
  output$nombrev1 <- renderPrint({
    cat(names(datasetInput1())[1])
  })
  
  output$nombrev2 <- renderPrint({
    cat(names(datasetInput2())[1])
  })
  
  output$nombrev3 <- renderPrint({
    cat(names(datasetInput1())[1])
  }) 
  
  output$nombrev4 <- renderPrint({
    cat(names(datasetInput2())[1])
  }) 
  
  output$nombrev5 <- renderPrint({
    cat(names(datasetInput1())[1])
  }) 
  
  output$nombrev6 <- renderPrint({
    cat(names(datasetInput2())[1])
  }) 
  
  
  ########################### DOS VARIABLES CATEGORICAS -- COMPONENTE 3 ####################################
  
  output$view1 <- renderTable({
    tabla <- ftable(datasetInput1()[,1],datasetInput2()[,1])
    tabla <- addmargins(tabla)
    tabla <- as.data.frame(tabla)
    nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
    nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
    tabla <- cbind(c(levels(nombresV1),"total"),tabla)
    colnames(tabla) <- c(" ",levels(nombresV2),"total")
    tabla
  })
  
  output$marg11 <- renderTable({
      fi <- table(datasetInput1()[,1])
      fr <- round(prop.table(table(datasetInput1()[,1])),3)
      Porcentaje <- round(fr,3)*100
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","fi","fr","Porcentaje")
      tabla
  })
  
  output$marg21 <- renderTable({
      fi <- table(datasetInput2()[,1])
      fr <- round(prop.table(table(datasetInput2()[,1])),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","fi","fr","Porcentaje")
      tabla
  })
  
  output$condX1 <- renderTable({
      tabla <- ftable(datasetInput1()[,1],datasetInput2()[,1])
      tabla <- prop.table(tabla,2)
      tabla <- data.frame(matrix(tabla, nrow = length(unique(datasetInput1()[,1]))))
      nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
      nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
      tabla <- cbind(c(levels(nombresV1)),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2))
      tabla
  })
  
  output$condY1 <- renderTable({
      tabla <- ftable(datasetInput1()[,1],datasetInput2()[,1])
      tabla <- prop.table(tabla,1)
      tabla <- data.frame(matrix(tabla, nrow = length(unique(datasetInput1()[,1]))))
      nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
      nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
      tabla <- cbind(c(levels(nombresV1)),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2))
      tabla
  })
  
  output$barras <- renderPlot({
      a <- data.frame(datasetInput1()[,1],datasetInput2()[,1])
      ggplot(a, aes(x=a[,1], fill=a[,2])) + geom_bar(position="dodge") + labs(x = names(datasetInput1())[1], fill = names(datasetInput2())[1], y = " ") + theme_bw() + theme(panel.background = element_blank ())
  })
  
  output$barrasApiladas <- renderPlot({
      a <- data.frame(datasetInput1()[,1],datasetInput2()[,1])
      ggplot(a, aes(x=a[,1], fill=a[,2])) + geom_bar(position="fill") + labs(x = names(datasetInput1())[1], fill = names(datasetInput2())[1], y = " ")+scale_y_continuous(labels=percent)
  })
  
  
  ########################### DOS VARIABLES NUMERICAS -- COMPONENTE 4 ####################################
  
  output$particionIntervalo1 <- renderText({
    if(input$tipoNumerica1 =="Continua" || input$tipoNumerica2 == "Continua"){
      "Escoger número de intervalos para la o las variables continuas:"
    }  
  })

  output$view <- renderTable({
    if(input$tipoNumerica1 == "Discreta" && input$tipoNumerica2 == "Discreta"){
      tabla <- ftable(datasetInput1()[,1],datasetInput2()[,1])
      tabla <- addmargins(tabla)
      tabla <- as.data.frame(tabla)
      nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
      nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
      tabla <- cbind(c(levels(nombresV1),"total"),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2),"total")
      tabla
    }else if(input$tipoNumerica1 == "Continua" && input$tipoNumerica2 == "Continua"){
      a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
      b <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
      tabla <- ftable(a,b)
      tabla <- addmargins(tabla)
      tabla <- as.data.frame(tabla)
      nombresV1 <- as.factor(unique(sort(a)))
      nombresV2 <- as.factor(unique(sort(b)))
      tabla <- cbind(c(levels(nombresV1),"total"),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2),"total")
      tabla
    }else if(input$tipoNumerica1 == "Discreta" && input$tipoNumerica2 == "Continua"){
    b <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
    tabla <- ftable(datasetInput1()[,1],b)
    tabla <- addmargins(tabla)
    tabla <- as.data.frame(tabla)
    nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
    nombresV2 <- as.factor(unique(sort(b)))
    tabla <- cbind(c(levels(nombresV1),"total"),tabla)
    colnames(tabla) <- c(" ",levels(nombresV2),"total")
    tabla
    }
    else if(input$tipoNumerica1 == "Continua" && input$tipoNumerica2 == "Discreta"){
      a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
      tabla <- ftable(a,datasetInput2()[,1])
      tabla <- addmargins(tabla)
      tabla <- as.data.frame(tabla)
      nombresV1 <- as.factor(unique(sort(a)))
      nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
      tabla <- cbind(c(levels(nombresV1),"total"),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2),"total")
      tabla
    }
    else{NULL}
  
  })
  
  output$marg1 <- renderTable({
    if(input$tipoNumerica1 == "Discreta"){
      fi <- table(datasetInput1()[,1])
      fr <- round(prop.table(table(datasetInput1()[,1])),3)
      Porcentaje <- round(fr,3)*100
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","fi","fr","Porcentaje")
      tabla
    }else if(input$tipoNumerica1 == "Continua"){
      a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
      fi <- (table(a))
      fr <- round(prop.table(table(a)),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","fi","fr","Porcentaje")
      tabla
    }
  })
  
  output$marg2 <- renderTable({
    if(input$tipoNumerica2 == "Discreta"){
      fi <- table(datasetInput2()[,1])
      fr <- round(prop.table(table(datasetInput2()[,1])),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","fi","fr","Porcentaje")
      tabla
    }else if(input$tipoNumerica2 == "Continua"){
      a <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
      fi <- (table(a))
      fr <- round(prop.table(table(a)),3)
      Porcentaje <- round(fr*100,4)
      tabla <- data.frame(fi,fr,Porcentaje)
      tabla <- tabla[,-c(3,5)]
      colnames(tabla) <- c("Datos","fi","fr","Porcentaje")
      tabla
    }
  })
  
  output$condX <- renderTable({
      if(input$tipoNumerica1 == "Discreta" && input$tipoNumerica2 == "Discreta"){
        tabla <- ftable(datasetInput1()[,1],datasetInput2()[,1])
        tabla <- prop.table(tabla,2)
        tabla <- data.frame(matrix(tabla, nrow = length(unique(datasetInput1()[,1]))))
        nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
        nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
        tabla <- cbind(c(levels(nombresV1)),tabla)
        colnames(tabla) <- c(" ",levels(nombresV2))
        tabla
        
      }else if(input$tipoNumerica1 == "Continua" && input$tipoNumerica2 == "Continua"){
        a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
        b <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
        tabla <- ftable(a,b)
        tabla <- prop.table(tabla,2)
        tabla <- data.frame(matrix(tabla, nrow = length(unique(a))))
        nombresV1 <- as.factor(unique(sort(a)))
        nombresV2 <- as.factor(unique(sort(b)))
        tabla <- cbind(c(levels(nombresV1)),tabla)
        colnames(tabla) <- c(" ",levels(nombresV2))
        tabla
      }else if(input$tipoNumerica1 == "Discreta" && input$tipoNumerica2 == "Continua"){
        b <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
        tabla <- ftable(datasetInput1()[,1],b)
        tabla <- prop.table(tabla,2)
        tabla <- data.frame(matrix(tabla, nrow = length(unique(datasetInput1()[,1]))))
        nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
        nombresV2 <- as.factor(unique(sort(b)))
        tabla <- cbind(c(levels(nombresV1)),tabla)
        colnames(tabla) <- c(" ",levels(nombresV2))
        tabla
      }
      else if(input$tipoNumerica1 == "Continua" && input$tipoNumerica2 == "Discreta"){
        a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
        tabla <- ftable(a,datasetInput2()[,1])
        tabla <- prop.table(tabla,2)
        tabla <- data.frame(matrix(tabla, nrow = length(unique(a))))
        nombresV1 <- as.factor(unique(sort(a)))
        nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
        tabla <- cbind(c(levels(nombresV1)),tabla)
        colnames(tabla) <- c(" ",levels(nombresV2))
        tabla
      }
      else{NULL}
      
  })
  
  output$condY <- renderTable({
    if(input$tipoNumerica1 == "Discreta" && input$tipoNumerica2 == "Discreta"){
      tabla <- ftable(datasetInput1()[,1],datasetInput2()[,1])
      tabla <- prop.table(tabla,1)
      tabla <- data.frame(matrix(tabla, nrow = length(unique(datasetInput1()[,1]))))
      nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
      nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
      tabla <- cbind(c(levels(nombresV1)),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2))
      tabla
      
    }else if(input$tipoNumerica1 == "Continua" && input$tipoNumerica2 == "Continua"){
      a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
      b <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
      tabla <- ftable(a,b)
      tabla <- prop.table(tabla,1)
      tabla <- data.frame(matrix(tabla, nrow = length(unique(a))))
      nombresV1 <- as.factor(unique(sort(a)))
      nombresV2 <- as.factor(unique(sort(b)))
      tabla <- cbind(c(levels(nombresV1)),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2))
      tabla
    }else if(input$tipoNumerica1 == "Discreta" && input$tipoNumerica2 == "Continua"){
      b <- factor(cut(datasetInput2()[,1],breaks = input$particionV2))
      tabla <- ftable(datasetInput1()[,1],b)
      tabla <- prop.table(tabla,1)
      tabla <- data.frame(matrix(tabla, nrow = length(unique(datasetInput1()[,1]))))
      nombresV1 <- as.factor(unique(sort(datasetInput1()[,1])))
      nombresV2 <- as.factor(unique(sort(b)))
      tabla <- cbind(c(levels(nombresV1)),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2))
      tabla
    }
    else if(input$tipoNumerica1 == "Continua" && input$tipoNumerica2 == "Discreta"){
      a <- factor(cut(datasetInput1()[,1],breaks = input$particionV1))
      tabla <- ftable(a,datasetInput2()[,1])
      tabla <- prop.table(tabla,1)
      tabla <- data.frame(matrix(tabla, nrow = length(unique(a))))
      nombresV1 <- as.factor(unique(sort(a)))
      nombresV2 <- as.factor(unique(sort(datasetInput2()[,1])))
      tabla <- cbind(c(levels(nombresV1)),tabla)
      colnames(tabla) <- c(" ",levels(nombresV2))
      tabla
    }
    else{NULL}
  })
  
  output$tablaIntermedia <- renderTable({
      if(input$resultados == "Mostrar"){
        Xi <- datasetInput1()[,1]
        Yi <- datasetInput2()[,1]
        Xi2 <- Xi^2
        Yi2 <- Yi^2
        XiYi <- Xi*Yi
        tabla <- data.frame(cbind(Xi,Yi,Xi2,Yi2,XiYi))
        names(tabla) <- c("X","Y","Cuadrado de X","Cuadrado de Y","Producto X por Y")
        tabla
      }else{NULL}
  })
  
  output$mediaX <- renderPrint({
      cat(summary(datasetInput1()[,1])[4]) 
  })
  
  output$mediaY <- renderPrint({
      cat(summary(datasetInput2()[,1])[4])
  })
  
  output$desviacionX <- renderPrint({
      n <- length(datasetInput1()[,1])
      desv <- sd(datasetInput1()[,1])*(n-1)/n
      cat(desv)
  })
  
  output$desviacionY <- renderPrint({
      n <- length(datasetInput2()[,1])
      desv <- sd(datasetInput2()[,1])*(n-1)/n
      cat(desv)
  })
  
  output$cov <- renderPrint({
      covarianza <- cov(datasetInput1()[,1],datasetInput2()[,1])
      cat(covarianza)
  })
  
  output$cor <- renderPrint({
      correlacion <- cor(datasetInput1()[,1],datasetInput2()[,1])
      cat(correlacion)
  })
  
  output$nube <- renderPlot({
      plot(datasetInput1()[,1],datasetInput2()[,1],col = 2, pch= 16,ylab= "Variable Y", xlab="Variable X")
  })
  
  output$regresionX <- renderPrint({
      regresion <- lm(datasetInput2()[,1]~datasetInput1()[,1])
      a <- data.frame(summary(regresion)[4])[1]
      rownames(a) <- c("Término independiente","Pendiente")
      colnames(a) <- c(" ")
      a
  })
  
  output$rectaX <- renderPlot({
      plot(datasetInput1()[,1],datasetInput2()[,1],col = 2, pch= 16,ylab= "Variable Y", xlab="Variable X")
      regresion <- lm(datasetInput2()[,1]~datasetInput1()[,1])
      abline(regresion)
  })
  
  output$rectaXModif <- renderPlot({
    plot(datasetInput1()[,1],datasetInput2()[,1],col = 2, pch= 16,ylab= "Variable Y", xlab="Variable X",xlim=c(input$infX,input$supX),ylim=c(input$infY,input$supY))
    regresion <- lm(datasetInput2()[,1]~datasetInput1()[,1])
    abline(regresion)
  })
  
  ########################### UNA VARIABLE NUMERICA Y UNA CATEGORICA -- COMPONENTE 5 #################################### 
  
  output$cajas <- renderPlot({
    if(input$orientacion == "Horizontal"){
      horizontal = T
    }else if(input$orientacion == "Vertical"){
      horizontal = F
    }
    if(input$tipo1 == "Cualitativa" && input$tipo2 == "Numérica"){
      boxplot(datasetInput2()[,1]~datasetInput1()[,1],horizontal=horizontal,col = "blue")
    }else if(input$tipo2 == "Cualitativa" && input$tipo1 == "Numérica"){
      boxplot(datasetInput1()[,1]~datasetInput2()[,1],horizontal=horizontal,col = "blue")
    }else{NULL}
  })
  
  output$estadisticos <- renderTable({
    if(input$tipo1 == "Cualitativa" && input$tipo2 == "Numérica"){
      a <- unique(datasetInput1()[,1])
      col <- matrix(NA,nrow = length(datasetInput2()[,1]),ncol = length(a))
      for(i in 1:length(a)){
        for(j in 1:length(datasetInput2()[,1])){
          if(datasetInput1()[j,1] == a[i]){
            col[j,i] <- datasetInput2()[j,1]
          }
        }
      }
      minimo <- rep(0,length(a))
      maximo <- rep(0,length(a))
      media <- rep(0,length(a))
      mediana <- rep(0,length(a))
      PrimerCuartil <- rep(0,length(a))
      TercerCuartil <- rep(0,length(a))
      for(i in 1:length(a)){
        minimo[i] <- min(na.omit(col[,i]))
        maximo[i] <- max(na.omit(col[,i]))
        media[i] <- round(mean(na.omit(col[,i])),2)
        mediana[i] <- median(na.omit(col[,i]))
        PrimerCuartil[i] <- quantile(na.omit(col[,i]), prob=c(0.25))
        TercerCuartil[i] <- quantile(na.omit(col[,i]), prob=c(0.75))
      }
      nombres <-data.frame(unique(datasetInput1()[,1]))
      tabla <- cbind(nombres,minimo,maximo,media,mediana,PrimerCuartil,TercerCuartil)
      tabla <- data.frame(tabla)
      colnames(tabla)<- c("Variable cualtitativa","Min","Max","Media","Mediana","Primer Cuartil","Tercer Cuartil")
      tabla
    }
    else if(input$tipo2 == "Cualitativa" && input$tipo1 == "Numérica"){
      a <-unique(datasetInput2()[,1])
      col <- matrix(NA,nrow = length(datasetInput1()[,1]),ncol = length(a))
      for(i in 1:length(a)){
        for(j in 1:length(datasetInput1()[,1])){
          if(datasetInput2()[j,1] == a[i]){
            col[j,i] <- datasetInput1()[j,1]
          }
        }
      }
      minimo <- rep(0,length(a))
      maximo <- rep(0,length(a))
      media <- rep(0,length(a))
      mediana <- rep(0,length(a))
      PrimerCuartil <- rep(0,length(a))
      TercerCuartil <- rep(0,length(a))
      for(i in 1:length(a)){
        minimo[i] <- min(na.omit(col[,i]))
        maximo[i] <- max(na.omit(col[,i]))
        media[i] <- round(mean(na.omit(col[,i])),2)
        mediana[i] <- median(na.omit(col[,i]))
        PrimerCuartil[i] <- quantile(na.omit(col[,i]), prob=c(0.25))
        TercerCuartil[i] <- quantile(na.omit(col[,i]), prob=c(0.75))
      }
      nombres <- data.frame(unique(datasetInput2()[,1]))
      tabla <- cbind(nombres,minimo,maximo,media,mediana,PrimerCuartil,TercerCuartil)
      tabla <- data.frame(tabla)
      colnames(tabla)<- c("Variable cualtitativa","Min","Max","Media","Mediana","Primer Cuartil","Tercer Cuartil")
      tabla
    }else{NULL}
  })
  
  ########################################################################################################
  
}
shinyApp(ui = ui, server = server)